<?php
/**
* 2014 Apple Inc.
*
* NOTICE OF LICENSE
*
* This source file is subject to the Open Software License (OSL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/osl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to telco.csee@geopost.pl so we can send you a copy immediately.
*
*  @author    JSC INVERTUS www.invertus.lt <help@invertus.lt>
*  @copyright 2014 DPD Polska sp. z o.o.
*  @license   http://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
*  International Registered Trademark & Property of DPD Polska sp. z o.o.
*/

if (!defined('_PS_VERSION_'))
	exit;

class DpdGeopostWS extends DpdGeopostController
{
	private $client; /* NUSoap client */
	protected $config;
	private $endpoint;
	private $params = array();
	private $list = array();
	private $listName;
	private $payload;
	private $lastCalledFunctionName;
	private $lastCalledFunctionArgs = array();

	protected $targetNamespace;
	protected $serviceName;

	const	applicationType = 9;
	const 	debug 			= true;
	const 	FILENAME 		= 'dpdgeopost.ws';

	const 	DEBUG_FILENAME			= 'DPDGEOPOST_DEBUG_FILENAME';
	const 	DEBUG_POPUP				= false;
	const 	DEBUG_FILENAME_LENGTH 	= 16;

	public function __construct()
	{
		parent::__construct();
		$this->config = new DpdGeopostConfiguration;
	}

	public function __call($name, $arguments)
	{
		if (!class_exists('nusoap_base'))
			require_once dirname(__FILE__).'/nusoap/nusoap.php';

		$this->lastCalledFunctionName = $name;
		$this->lastCalledFunctionArgs = $arguments;

		if ($this->loadWSData())
		{
			if (isset($arguments[0]) && is_string($arguments[0]))
				$this->listName = $arguments[0];
			else
				$this->listName = null;

			if (isset($arguments[2]) && is_array($arguments[2]))
				$this->params = array_merge($this->params, $arguments[2]);

			$this->endpoint .= $this->serviceName;
			$this->client = new nusoap_client($this->endpoint, false, false, false, false, (int)$this->config->ws_timeout);
			$this->client->soap_defencoding = 'UTF-8';

			if ($error = $this->client->getError())
				self::$errors[] = $error;
		}

		if (count(self::$errors))
		{
			if (self::debug)
				$this->debug();
			return false;
		}

		if (isset($arguments[1]) && is_array($arguments[1]))
		{
			$this->list = array_merge($arguments[1], array(
				'payerId' => (int)$this->config->payer_id,
				'senderAddressId' => (int)$this->config->sender_id
			));

			$this->createPayload();

			$result = $this->client->call($name, $this->payload, $this->targetNamespace);

			if (!$result)
				return array('prestashop_message' => $this->l('Connection refused by web services.'));

			if ($error = $this->getError($result))
				self::$errors[] = utf8_encode($error);

			if (self::debug)
				$this->debug($result);

			return $result;
		}

		return false;
	}

	private function createPayload()
	{
		$this->payload = '';

		foreach($this->params as $param_name => $param_value)
			$this->payload .= $this->client->serialize_val($param_value, $param_name, false, false, false, false, 'encoded');

		if ($this->list)
		{
			if ($this->listName)
				$this->payload .= "<{$this->listName}>";

			foreach($this->list as $param_name => $param_value)
			{
				if (!$param_name)
					foreach ($param_value['data'] as $subparam_value)
						$this->payload .= $this->client->serialize_val($subparam_value, $param_value['name'], false, false, false, false, 'encoded');
				else
					$this->payload .= $this->client->serialize_val($param_value, $param_name, false, false, false, false, 'encoded');
			}

			if ($this->listName)
			{
				$this->payload .= "</{$this->listName}>";
				$this->params[$this->listName] = $this->list;
			}
		}
	}

	private function loadWSData()
	{
		if ($this->loadEndpoint())
		{
			if ($this->config->ws_username && $this->config->ws_password)
			{
				$this->params = array(
					'wsUserName' => pSQL($this->config->ws_username),
					'wsPassword' => pSQL($this->config->ws_password),
					'wsLang' => pSQL($this->context->language->iso_code),
					'applicationType' => pSQL(self::applicationType)
				);

				return true;
			}
			else
				self::$errors[] = $this->l('WS username / password is missing');
		}

		return false;
	}

	private function loadEndpoint()
	{
		if (!$this->config->dpd_country_select)
		{
			self::$errors[] = $this->l('DPD country is not selected.');
			return false;
		}
		elseif ($this->config->dpd_country_select == DpdGeopostConfiguration::OTHER)
		{
			if ($this->config->production_mode)
			{
				if ($this->config->ws_production_url)
					$this->endpoint = $this->config->ws_production_url;
				else
				{
					self::$errors[] = $this->l('Production WS URL is missing.');
					return false;
				}
			}
			else
			{
				if ($this->config->ws_test_url)
					$this->endpoint = $this->config->ws_test_url;
				else
				{
					self::$errors[] = $this->l('Test WS URL is missing.');
					return false;
				}
			}
		}
		elseif (!isset($this->config->countries[$this->config->dpd_country_select]))
		{
			self::$errors[] = $this->l('Selected DPD country is not supported.');
			return false;
		}
		else
		{
			if ($this->config->production_mode)
			{
				if (isset($this->config->countries[$this->config->dpd_country_select]['ws_uri_prod']) && $this->config->countries[$this->config->dpd_country_select]['ws_uri_prod'])
					$this->endpoint = $this->config->countries[$this->config->dpd_country_select]['ws_uri_prod'];
				else
				{
					self::$errors[] = $this->l('Missing production WS URL for your selected DPD country.');
					return false;
				}
			}
			else
			{
				if (isset($this->config->countries[$this->config->dpd_country_select]['ws_uri_test']) && $this->config->countries[$this->config->dpd_country_select]['ws_uri_test'])
					$this->endpoint = $this->config->countries[$this->config->dpd_country_select]['ws_uri_test'];
				else
				{
					self::$errors[] = $this->l('Missing test WS URL for your selected DPD country.');
					return false;
				}
			}
		}
		return true;
	}

	protected function getError($result)
	{
		$transaction_id = isset($result['transactionId']) ? '. '.$this->module_instance->l('Transaction Id:').' '.$result['transactionId'] : '';

		if (isset($result['detail']))
			return $result['detail']['EShopException']['error']['text'].$transaction_id;

		if (isset($result['priceList']['error']['text']))
			return $result['priceList']['error']['text'].$transaction_id;

		if (isset($result['resultList']['error']['text']))
			return $result['resultList']['error']['text'].$transaction_id;

		if (isset($result['error']['text']))
			return $result['error']['text'].$transaction_id;

		if (isset($result['prestashop_message']))
			return $result['prestashop_message'].$transaction_id;

		return null;
	}

	private function createDebugFileIfNotExists()
	{
		if ((!$debug_filename = Configuration::get(self::DEBUG_FILENAME)) || !$this->isDebugFileName($debug_filename))
		{
			$debug_filename = Tools::passwdGen(self::DEBUG_FILENAME_LENGTH).'.html';
			Configuration::updateValue(self::DEBUG_FILENAME, $debug_filename);
		}

		if (!file_exists(_DPDGEOPOST_MODULE_DIR_.$debug_filename))
		{
			$file = fopen(_DPDGEOPOST_MODULE_DIR_.$debug_filename, 'w');
			fclose($file);
		}

		return $debug_filename;
	}

	private function isDebugFileName($debug_filename)
	{
		return Tools::strlen($debug_filename) == (int)self::DEBUG_FILENAME_LENGTH + 5 && preg_match('#^[a-zA-Z0-9]+\.html$#', $debug_filename);
	}

	private function debug($result = null)
	{
		$debug_html = '';

		if ($this->lastCalledFunctionName)
		{
			$debug_html .= '<h2 style="padding: 10px 0 10px 0; display: block; border-top: solid 2px #000000; border-bottom: solid 2px #000000;">
			['.date('Y-m-d H:i:s').']</h2><h2>Function \''.$this->lastCalledFunctionName.'\' params
			</h2><pre>';
			$debug_html .= print_r($this->lastCalledFunctionArgs, true);
			$debug_html .= '</pre>';
		}

		if ($this->payload)
			$debug_html .= '<h2>Request</h2><pre>' . $this->displayPayload() . '</pre>';

		if ($result)
		{
			if ($err = $this->client->getError())
				$debug_html .= '<h2>Error</h2><pre>' . $err . '</pre>';
			else
			{
				$debug_html .= '<h2>Response</h2><pre>';
				$debug_html .= print_r($result, true);
				$debug_html .= '</pre>';
			}
		}
		else
		{
			$debug_html .= '<h2>Errors</h2><pre>' . print_r(self::$errors, true) . '</pre>';
		}

		if ($debug_html)
		{
			$debug_filename = $this->createDebugFileIfNotExists();

			$current_content = Tools::file_get_contents(_DPDGEOPOST_MODULE_DIR_.$debug_filename);
			@file_put_contents(_DPDGEOPOST_MODULE_DIR_.$debug_filename, $debug_html.$current_content, LOCK_EX);

			if (self::DEBUG_POPUP)
			{
				echo '
				<div id="_invertus_ws_console" style="display:none">
					'.$debug_html.'
				</div>
				<script language=javascript>
					_invertus_ws_console = window.open("","Invertus WS debugging console","width=680,height=600,resizable,scrollbars=yes");
					_invertus_ws_console.document.write("<HTML><TITLE>Invertus WS debugging console</TITLE><BODY bgcolor=#ffffff>");
					_invertus_ws_console.document.write(document.getElementById("_invertus_ws_console").innerHTML);
					_invertus_ws_console.document.write("</BODY></HTML>");
					_invertus_ws_console.document.close();
					document.getElementById("_invertus_ws_console").remove();
				</script>';
			}
		}
	}

	/* only for debugging purposes */
	private function displayPayload()
	{
		$xml = preg_replace('/(>)(<)(\/*)/', "$1\n$2$3", $this->payload);
		$token	  = strtok($xml, "\n");
		$result	 = '';
		$pad		= 0;
		$matches	= array();
		while ($token !== false) :
			if (preg_match('/.+<\/\w[^>]*>$/', $token, $matches)) :
			  $indent=0;
			elseif (preg_match('/^<\/\w/', $token, $matches)) :
			  $pad-=4;
			  $indent = 0;
			elseif (preg_match('/^<\w[^>]*[^\/]>.*$/', $token, $matches)) :
			  $indent = 4;
			else :
			  $indent = 0;
			endif;
			$line	= str_pad($token, Tools::strlen($token)+$pad, ' ', STR_PAD_LEFT);
			$result .= $line . "\n";
			$token   = strtok("\n");
			$pad	+= $indent;
		endwhile;

		return htmlentities($result);
	}
}